'use strict';
/**
 * node-windows-window-controller
 *
 * @author waiting
 * @license MIT
 * @link https://github.com/waitingsong/node-windows-window-controller
 */
Object.defineProperty(exports, '__esModule', { value: true });
const u32 = require('./lib/user32');
var user32_1 = require('./lib/user32');
exports.validate_cmdshow = user32_1.validate_cmdshow;
const isWin32 = process.platform === 'win32';
const plateformError = 'Invalid platform: win32 required';
// expose original user32.ShowWindow() as default
function showWindow(hWnd, nCmdShow) {
    let errcode = 1;
    if (!Number.isSafeInteger(hWnd)) {
        console.error('hWnd must integer');
        return errcode;
    }
    if (!u32.validate_cmdshow(nCmdShow)) {
        return errcode;
    }
    try {
        if (u32.api.ShowWindow(hWnd, nCmdShow)) {
            errcode = 0;
        }
    }
    catch (ex) {
        console.error(ex);
    }
    return errcode;
}
exports.default = showWindow;
function hide(p, onlyMainWin = true) {
    return proxy(p, u32.CmdShow.SW_HIDE, onlyMainWin);
}
exports.hide = hide;
function show(p, nCmdShow, onlyMainWin = true) {
    return proxy(p, nCmdShow, onlyMainWin);
}
exports.show = show;
// https://msdn.microsoft.com/en-us/library/windows/desktop/ms633548(v=vs.85).aspx
function proxy(p, nCmdShow, onlyMainWin) {
    const execRet = init_execret();
    if (!u32.validate_cmdshow(nCmdShow)) {
        execRet.err = 1;
        execRet.msg = 'value of nCmdShow invalid';
        return Promise.resolve(execRet);
    }
    return new Promise(resolve => {
        if (onlyMainWin) {
            get_main_hwnd(p).then(hWnd => {
                if (hWnd && typeof hWnd === 'number') {
                    u32.show(hWnd, nCmdShow, onlyMainWin)
                        .then((hWnd) => {
                        hWnd && execRet.hwnds.push(hWnd);
                    })
                        .catch(err => {
                        execRet.err = 1;
                        execRet.msg += '\n ' + err;
                    })
                        .then(() => {
                        resolve();
                    });
                }
                else {
                    resolve();
                }
            });
        }
        else {
            get_hwnds(p).then(arr => {
                if (arr && arr.length) {
                    Promise.all(arr.map(hWnd => {
                        return u32.show(hWnd, nCmdShow, onlyMainWin)
                            .then((hWnd) => hWnd && execRet.hwnds.push(hWnd))
                            .catch(err => {
                            execRet.err = 1;
                            execRet.msg += '\n ' + err;
                        });
                    }))
                        .then(resolve);
                }
                else {
                    resolve();
                }
            });
        }
    }).then(() => execRet);
}
function get_main_hwnd(p) {
    return get_hwnds(p).then(arr => {
        if (arr && Array.isArray(arr) && arr.length) {
            return u32.filter_main_hwnd(arr);
        }
    });
}
exports.get_main_hwnd = get_main_hwnd;
function get_hwnds(p) {
    return u32.get_hwnds(p);
}
exports.get_hwnds = get_hwnds;
// kill process by which the matched hWnd(s) (window) created
function kill(p) {
    const execRet = init_execret();
    if (typeof p === 'number') {
        _kill(p);
        execRet.pids.push(p);
        return Promise.resolve(execRet);
    }
    else if (typeof p === 'string') {
        return new Promise(resolve => {
            const task = u32.create_task();
            return u32.get_hwnds(p, task).then(() => {
                if (task.pidSet.size) {
                    for (let pid of task.pidSet) {
                        if (_kill(pid)) {
                            execRet.pids.push(pid);
                        }
                    }
                }
                else {
                    execRet.msg = 'the pid list to be killed empty';
                }
                resolve(execRet);
            });
        });
    }
    else {
        execRet.err = 1;
        execRet.msg = 'kill() param must be either number or string';
        return Promise.resolve(execRet);
    }
}
exports.kill = kill;
function _kill(pid) {
    try {
        process.kill(pid, 0) && process.kill(pid);
        return true;
    }
    catch (ex) {
        console.error(ex);
    }
    return false;
}
function init_execret() {
    return {
        err: 0,
        msg: '',
        pids: [],
        hwnds: [],
    };
}
